﻿Type=Class
Version=3.61
ModulesStructureVersion=1
B4J=true
@EndOfDesignText@
'Class module
Sub Class_Globals
	Private fx As JFX
	Public gv As GameView
	Private mainTimer As Timer
	Public Asteroids As List
	Private AsteroidsBitmaps(,) As BitmapData
	Private MyShip As Ship
	Public missileBitmap() As Image
	Public missileRect(2) As Rect
	Public missiles As List
	Public shotSoundId, breakSoundId, crashSoundId As Int
	Public GameViewHelper As GameViewHelper
	Public ArrowKeys As Map
	Private TimerInterval As Int
	Public TimeScale As Double
End Sub

Public Sub Initialize (vGV As GameView)
	ArrowKeys = CreateMap("Left": False, "Up": False, "Right": False, "Down": False, "Space": False)
	gv = vGV
#if DEBUG
	TimerInterval = 60
#else
	TimerInterval = 8
#end if
	TimeScale = TimerInterval / 30
	mainTimer.Initialize("MainTimer", TimerInterval)
	AsteroidsBitmaps = GameUtils.LoadSpritesFromSheet(fx.LoadImage(File.DirAssets, "asteroids.png"), 4, 4)
	Asteroids.Initialize
	missileBitmap = Array As Image(fx.LoadImage(File.DirAssets, "missile1.png"), _ 
		fx.LoadImage(File.DirAssets, "missile2.png"))
	gv.BitmapsData.Add(CreateBackground)
	Dim bd As BitmapData
	bd.Initialize
	bd.Tag = "Ship"
	MyShip.Initialize(bd , Me)
	gv.BitmapsData.Add(bd)
	missiles.Initialize
	shotSoundId = GameViewHelper.LoadAudioClip(File.GetUri(File.DirAssets, "shotgun.mp3"))
	breakSoundId = GameViewHelper.LoadAudioClip(File.GetUri(File.DirAssets, "break.mp3"))
	crashSoundId = GameViewHelper.LoadAudioClip(File.GetUri(File.DirAssets, "crash.mp3"))
	GameViewHelper.AddKeyListener("GVH", Main.MainForm)
	GameViewHelper.SetBoxBlur(gv.AsNode)
	StartGame
End Sub

Sub GVH_KeyPressed (KeyCode As String) As Boolean
	If ArrowKeys.ContainsKey(KeyCode) Then 
		ArrowKeys.Put(KeyCode, True)
		Return True
	Else
		Return False
	End If
End Sub

Sub GVH_KeyReleased (KeyCode As String) As Boolean
	If KeyCode = "Space" Then Return True 'we don't want to miss a short space press.
	If ArrowKeys.ContainsKey(KeyCode) Then 
		ArrowKeys.Put(KeyCode, False)
		Return True
	Else
		Return False
	End If
End Sub

Private Sub StartGame
	'Clear the previous remaining asetroids.
	For i = 0 To Asteroids.Size - 1
		Dim ast As Asteroid
		ast = Asteroids.Get(i)
		ast.RemoveFromGameView 'This method will remove the asteroid from the GameView
	Next
	Asteroids.Clear
	'Create 4 new asteroids
	For i = 1 To 4
		AddAsteroid (Rnd(10, gv.Width - 10), Rnd(10, gv.Height - 10), _
			Rnd(50 , 150))
	Next
	
	MyShip.ResetShip
End Sub

Public Sub AddAsteroid (x As Int, y As Int, size As Int)
	Dim animator As SpriteAnimator
	Dim rects(4) As Rect
	Dim bitmaps(4) As Image
	Dim row As Int
	row = Rnd(0, 4)
	'The asteroids sprite sheet includes 4 rows of sprites.
	'One of the rows is randomly selected.
	For i = 0 To 3
		rects(i) = AsteroidsBitmaps(i, row).SrcRect
		bitmaps(i) = AsteroidsBitmaps(i, row).Bitmap
	Next
	
	animator.Initialize(gv, size, size)
	Dim bd As BitmapData
	bd.Initialize
	animator.SetFrames(bd, rects, bitmaps) 'sets the frames that will be animated
	animator.AnimationInterval = 10 / TimeScale
	Dim ast As Asteroid
	ast.Initialize(bd, animator, Me, x, y, size)
	Asteroids.Add(ast) 'add the asteroid to the asteroids list.
	gv.BitmapsData.Add(bd) 'add the asteroid to the GameView (so it will be redrawn every time).
End Sub

Public Sub StartTimer
	mainTimer.Enabled = True
End Sub

Public Sub StopTimer
	mainTimer.Enabled = False
End Sub

Private Sub mainTimer_Tick
	'calculate the frames per second	
	Dim ast As Asteroid
	Dim i As Int
	i = 0
	'if there are no more asteroids then the game is completed (the player has destroyed all of them).
	If Asteroids.Size = 0 Then
		'Msgbox("Well done!", "")
		StartGame
		Return
	End If
	'go over all the asteroids and call their tick method.
	Do While i < Asteroids.Size
		ast = Asteroids.Get(i)
		'if the asteroid delete flag is on then we remove it from the list.
		If ast.delete Then
			Asteroids.RemoveAt(i)
		Else
			ast.Tick
			i = i + 1
		End If
	Loop
	'similar implementation for the missiles.
	Dim mis As Missile
	i = 0
	Do While i < missiles.Size
		mis = missiles.Get(i)
		If mis.delete Then
			missiles.RemoveAt(i) 'remove missile
		Else
			mis.Tick
			i = i + 1
		End If
	Loop
	'call the ship tick method.
	MyShip.Tick
	'check whether the ship is in the crash state.
	If MyShip.state = MyShip.STATE_CRASHED Then
		'Msgbox("Try again...", "")
		StartGame
	End If
	gv.Draw
End Sub

Public Sub PlaySound(sound As Int, volume As Float)
	GameViewHelper.PlayAudioClip(sound, volume)
End Sub

'Create the stars background
Private Sub CreateBackground As BitmapData
	'The background is created manually with a bitmap and canvas.
	Dim cvs As Canvas
	cvs.Initialize("")
	Main.MainForm.RootPane.AddNode(cvs, 0, 0, gv.Width, gv.Height)
	cvs.DrawRect(0, 0, cvs.Width, cvs.Height, fx.Colors.Black, True, 0)
	For i = 1 To 100
		cvs.DrawCircle(Rnd(0, gv.Width), Rnd(0, gv.Height), 2dip, fx.Colors.White, True, 0)
	Next
	Dim bd As BitmapData
	bd.Initialize
	bd.Bitmap = cvs.Snapshot
	bd.SrcRect = GameUtils.CreateRect(0, 0, bd.Bitmap.Width, bd.Bitmap.Height)
	bd.DestRect = GameUtils.CreateRect(0, 0, gv.Width, gv.Height)
	cvs.RemoveNodeFromParent
	Return bd
End Sub



