﻿Type=Class
Version=3.61
ModulesStructureVersion=1
B4J=true
@EndOfDesignText@
'Class module
Sub Class_Globals
	Private fx As JFX
	Private animator As SpriteAnimator
	Private shipBd As BitmapData
	Private game As GameManager
	Private MAX_SPEED As Double = 8
	Private speed As Float
	Private shipBmp, shipForwardBmp, shipBackwardBmp As Image
	Private currentBmp As Image
	Private shipRect As Rect
	Private ticks, lastFire, fireInterval = 5, explodeStart As Int
	Private gracePeriod = 180 As Int
	Private missileSpeed = 12  As Double
	Private collision As CollisionDetector
	Private crashBitmaps(,) As BitmapData
	Public state As Int
	Public const STATE_NORMAL = 0, STATE_EXPLODING = 1, STATE_CRASHED = 2 As Int
End Sub

Public Sub Initialize (vBD As BitmapData, vGame As GameManager)
	animator.Initialize (vGame.gv, 30, 45)
	shipBd = vBD
	game = vGame
	shipBmp = fx.LoadImage(File.DirAssets, "ship.png")
	shipForwardBmp = fx.LoadImage(File.DirAssets, "ship_forward.png")
	shipBackwardBmp = fx.LoadImage(File.DirAssets, "ship_back.png")
	
	gracePeriod = gracePeriod / game.TimeScale
	fireInterval = fireInterval / game.TimeScale
	MAX_SPEED = MAX_SPEED * game.TimeScale
	missileSpeed = missileSpeed * game.TimeScale
	
	currentBmp = shipBmp
	Dim x, y As Int
	x = vGame.gv.Width / 2 - animator.width / 2 'start in the middle of the screen
	y = vGame.gv.Height / 2 - animator.height / 2
	shipBd.DestRect = GameUtils.CreateRect(x, y ,  animator.width,  animator.height)
	shipBd.Flip = game.GameViewHelper.FLIP_NONE
	'initialize the ship's collistion detector. It will detect collisions between the ship and the asteroids.
	'passing a SafeZone value of 10 * GameUtils.scale makes the asteroids a bit less harmful.
	collision.Initialize(shipBd.DestRect, game.Asteroids, 10, 10)
	'load the bitmaps that will be used when the ship crashes.
	crashBitmaps = GameUtils.LoadSpritesFromSheet(fx.LoadImage(File.DirAssets, "crash.png"), 7, 1)
End Sub

Public Sub ResetShip
	ticks = 0
	state = STATE_NORMAL
	lastFire = -100
	animator.AnimationInterval = 1
	'note that the ship is drawn from a single frame.
	'shipRect is never initialized so GameView draws the complete bitmap.
	animator.SetFrames(shipBd, Array As Rect(shipRect), Array As Image(shipBmp))
End Sub

Public Sub Tick
	ticks = ticks + 1
	If state = STATE_NORMAL Then
		'check the left joystick and change the rotation and speed
		If game.ArrowKeys.Get("Left") = True Then
			shipBd.Rotate = shipBd.Rotate - 7 * game.TimeScale
		Else If game.ArrowKeys.Get("Right") = True Then
			shipBd.Rotate = shipBd.Rotate + 7 * game.TimeScale
		End If
		If game.ArrowKeys.Get("Up") = True Then
			speed = Min(speed + 0.2 * game.TimeScale, MAX_SPEED)
				animator.SetFrames(shipBd, Array As Rect(shipRect), Array As Image(shipForwardBmp))
				currentBmp = shipForwardBmp
		Else If game.ArrowKeys.Get("Down") = True Then
			speed = Max(speed - 0.2 * game.TimeScale, -MAX_SPEED)
			animator.SetFrames(shipBd, Array As Rect(shipRect), Array As Image(shipBackwardBmp))
			currentBmp = shipBackwardBmp
		Else
			If currentBmp <> shipBmp Then
				animator.SetFrames(shipBd, Array As Rect(shipRect), Array As Image(shipBmp))
				currentBmp = shipBmp
			End If
		End If
		
		If ticks - lastFire > fireInterval Then
			lastFire = ticks
			If game.ArrowKeys.Get("Space") = True Then
				ShootMissile(shipBd.Rotate)
				game.ArrowKeys.Put("Space", False)
			End If
		End If 
		'set the ship speed
		animator.vx = speed * SinD(shipBd.Rotate)
		animator.vy = -speed * CosD(shipBd.Rotate)
		
		'Calling Collision.Tick actually checks for collisions.
		If collision.Tick <> Null And ticks > gracePeriod Then
			Crash
		End If
		
	Else If state = STATE_EXPLODING Then
		If ticks - explodeStart > 120 Then
			state = STATE_CRASHED
		End If
	End If
	animator.Tick(shipBd) 'move the ship
End Sub

Private Sub Crash
	state = STATE_EXPLODING
	game.PlaySound(game.crashSoundId, 0.3)
	Dim rects(4) As Rect
	Dim bitmaps(4) As Image
	For i = 0 To 3
		rects(i) = crashBitmaps(i, 0).SrcRect
		bitmaps(i) = crashBitmaps(i, 0).Bitmap
	Next
	'Change the ship animation frames.
	animator.SetFrames(shipBd, rects, bitmaps)
	animator.AnimationInterval = 10
	explodeStart = ticks
End Sub

Private Sub ShootMissile (offset As Int)
	game.PlaySound(game.shotSoundId, 0.3)
	'create a new Missile and add it to GameView and GameManager.
	Dim msl As Missile
	Dim missileAnimator As SpriteAnimator
	missileAnimator.Initialize(game.gv, 4, 8)
	Dim mbd As BitmapData
	mbd.Initialize
	missileAnimator.SetFrames(mbd, game.missileRect, game.missileBitmap)
	mbd.Rotate = offset
	Dim x, y As Double
	x = shipBd.DestRect.CenterX
	y = shipBd.DestRect.CenterY
	mbd.DestRect = GameUtils.CreateRect(x, y, missileAnimator.width,  missileAnimator.height)
	msl.Initialize(mbd, missileAnimator, game)
	missileAnimator.vx = missileSpeed * SinD(offset)
	missileAnimator.vy = -missileSpeed * CosD(offset)
	game.gv.BitmapsData.Add(mbd)
	game.missiles.Add(msl)
End Sub